from . import group
from . import base64
from . import env
from . import eval
from . import git
from . import md5
from . import explain
from . import project
from . import ubuntu
from . import update
from . import version
from . import devtool
from . import generator

assert group
assert base64
assert env
assert eval
assert git
assert md5
assert explain
assert project
assert ubuntu
assert update
assert version
assert devtool
assert generator
